'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _output = require('./output');

var _output2 = _interopRequireDefault(_output);

var _parser = require('./parser');

var _parser2 = _interopRequireDefault(_parser);

var _package = require('../package.json');

var _package2 = _interopRequireDefault(_package);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _jestValidate = require('jest-validate');

var _http = require('./http');

var _http2 = _interopRequireDefault(_http);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Command extends _output2.default {
  constructor(...args) {
    var _temp;

    return _temp = super(...args), this.http = new _http2.default(this), _temp;
  }

  static get id() {
    return this.command ? `${this.topic}:${this.command}` : this.topic;
  }

  static async run(argv = [], options = {}) {
    let config = options.config || new _config2.default();
    if (options.mock) config.mock = true;
    let cmd = new this(config);
    cmd.argv = argv;
    cmd.validate();
    try {
      await cmd.init();
      await cmd.run();
      await cmd.done();
    } catch (err) {
      if (config.mock) throw err;
      cmd.error(err);
    }
    return cmd;
  }

  // prevent setting things that need to be static


  validate() {
    const exampleFlag = {
      name: 'app',
      char: 'a',
      hidden: false,
      hasValue: false,
      required: false,
      optional: true,
      description: 'description of flag',
      default: () => {}
    };
    const exampleArg = {
      name: 'FILE',
      required: true,
      optional: false,
      hidden: false
    };
    const id = this.constructor.id;
    (0, _jestValidate.validate)(this.constructor, {
      comment: `Command: ${id}`,
      exampleConfig: {
        topic: 'apps',
        command: 'info',
        description: 'description of my command',
        hidden: true,
        variableArgs: false,
        usage: 'how to use the command',
        help: 'long form help text',
        aliases: ['-v', '--version'],
        args: [exampleArg],
        flags: [exampleFlag],
        _flags: [exampleFlag]
      }
    });
    for (let flag of this.constructor.flags) {
      (0, _jestValidate.validate)(flag, {
        comment: `${id} flag: ${flag.name}`,
        exampleConfig: exampleFlag
      });
    }
    for (let arg of this.constructor.args) {
      (0, _jestValidate.validate)(arg, {
        comment: `${id} arg: ${arg.name}`,
        exampleConfig: exampleArg
      });
    }
  }

  async init() {
    await super.init();
    let parser = new _parser2.default(this);
    await parser.parse(this.constructor.args, this.constructor.flags, this.argv);
    this.flags = parser.flags;
    this.args = parser.args;
    this.argv = parser.argv;
    if (this.flags.debug) this.config.debug = 1;
  }

  /**
   * actual command run code goes here
   */
  async run() {}

  async done() {
    await super.done();
  }
}
exports.default = Command;
Command.aliases = [];
Command.variableArgs = false;
Command.flags = [];
Command.args = [];
Command._flags = [{ name: 'debug', hidden: true }, { name: 'no-color', hidden: true }];