'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function debug() {
  const HEROKU_DEBUG = process.env.HEROKU_DEBUG;
  if (HEROKU_DEBUG === 'true') return 1;
  if (HEROKU_DEBUG) return parseInt(HEROKU_DEBUG);
  return 0;
}

class Dirs {
  constructor(config) {
    this._config = config;
  }

  get home() {
    return _os2.default.homedir() || _os2.default.tmpdir();
  }
  get data() {
    return this._fetch('data');
  }
  get config() {
    return this._fetch('config');
  }
  get cache() {
    let def;
    if (process.platform === 'darwin') def = _path2.default.join(this.home, 'Library', 'Caches');
    return this._fetch('cache', def);
  }

  get _fs() {
    return require('fs-extra');
  }
  _mkdirp(dir) {
    this._fs.mkdirpSync(dir);
  }

  _fetch(category, d) {
    d = d || _path2.default.join(this.home, category === 'data' ? '.local/share' : '.' + category);
    if (this.windows) d = process.env.LOCALAPPDATA || d;
    d = process.env.XDG_DATA_HOME || d;
    d = _path2.default.join(d, this._config.name);
    this._mkdirp(d);
    return d;
  }

  toJSON() {
    return {
      cache: this.cache,
      data: this.data,
      config: this.config,
      home: this.home
    };
  }
}

class Config {
  constructor(options = {}) {
    this._options = options;
    this._pjson = this._options.root
    // flow$ignore
    ? require(_path2.default.join(options.root, 'package.json')) : require('../package.json');
    this.debug = debug() || options.debug || 0;
    this.dirs = new Dirs(this);
    this.mock = options.mock || false;
    this.argv = options.argv || [];
  }

  get name() {
    return this._pjson.name;
  }
  get version() {
    return this._options.version || this._pjson.version;
  }
  get channel() {
    return this._options.channel || 'stable';
  }
  get updateDisabled() {
    return this._options.updateDisabled;
  }
  get bin() {
    return this._cli.bin || this._pjson.name;
  }
  get root() {
    return this._options.root || _path2.default.join(__dirname, '..');
  }
  get defaultCommand() {
    return this._cli.defaultCommand || 'help';
  }
  get s3() {
    return this._cli.s3 || {};
  }
  get _cli() {
    return this._pjson['cli-engine'] || {};
  }
  get windows() {
    return _os2.default.platform === 'win32';
  }

  toJSON() {
    return {
      name: this.name,
      version: this.version,
      channel: this.channel,
      mock: this.mock,
      updateDisabled: this.updateDisabled,
      bin: this.bin,
      defaultCommand: this.defaultCommand,
      root: this.root,
      s3: this.s3,
      windows: this.windows,
      debug: this.debug,
      dirs: this.dirs
    };
  }
}
exports.default = Config;