'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

require('./arg');

require('./flag');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Parse {
  constructor(cmd) {
    this.flags = {};
    this.args = {};

    this.cmd = cmd;
  }

  async parse(args, flags, argv) {
    args = args.slice(0);
    flags = flags.slice(0);
    let parsingArgs = argv.slice(0);
    this.argv = [];

    async function parseFlags() {
      for (let flag of flags || []) {
        if (!this.flags[flag.name]) {
          if (flag.default) this.flags[flag.name] = await flag.default(this.cmd);
          if (!this.flags[flag.name] && (flag.optional === false || flag.required === true)) {
            throw new Error(`Missing required flag --${flag.name}`);
          }
        }
      }
    }

    let parseFlag = arg => {
      let long = arg.startsWith('--');
      let flag = long ? findLongFlag(arg) : findShortFlag(arg);
      if (!flag) return false;
      let cur = this.flags[flag.name];
      if (flag.hasValue) {
        if (cur) throw new Error(`Flag --${flag.name} already provided`);
        let val = long || arg.length < 3 ? parsingArgs.shift() : arg.slice(2);
        if (!val) throw new Error(`Flag --${flag.name} expects a value.`);
        this.flags[flag.name] = val;
      } else {
        if (!cur) this.flags[flag.name] = flag.name;
        // push the rest of the short characters back on the stack
        if (!long && arg.length > 2) parsingArgs.unshift(`-${arg.slice(2)}`);
      }
      return true;
    };

    let findLongFlag = arg => {
      return flags.find(f => f.name === arg.slice(2));
    };

    let findShortFlag = arg => {
      return flags.find(f => f.char === arg[1]);
    };

    let parsingFlags = true;
    while (parsingArgs.length) {
      let arg = parsingArgs.shift();
      if (parsingFlags && arg.startsWith('-')) {
        if (arg === '--') {
          parsingFlags = false;continue;
        }
        if (parseFlag(arg)) continue;
      }
      this.argv.push(arg);
      if (this.cmd.constructor.variableArgs) continue;
      let expected = args.shift();
      if (!expected) throw new Error(`Unexpected argument ${arg}`);
      this.args[expected.name] = arg;
    }

    let missingArg = args.find(a => a.optional !== true && a.required !== false);
    if (missingArg) throw new Error(`Missing required argument ${missingArg.name}`);

    await parseFlags.call(this);
  }
}
exports.default = Parse;